---
id: WebDriver
title: WebDriver
---
<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

## WebDriver

**Extends Helper**

WebDriver helper which wraps [webdriverio][1] library to
manipulate browser using Selenium WebDriver or PhantomJS.

WebDriver requires [Selenium Server and ChromeDriver/GeckoDriver to be installed][2].

### Configuration

This helper should be configured in codecept.json or codecept.conf.js

-   `url`: base url of website to be tested.
-   `browser`: browser in which to perform testing.
-   `host`: (optional, default: localhost) - WebDriver host to connect.
-   `port`: (optional, default: 4444) - WebDriver port to connect.
-   `protocol`: (optional, default: http) - protocol for WebDriver server.
-   `path`: (optional, default: /wd/hub) - path to WebDriver server,
-   `remoteFileUpload`: (optional, default: true) - upload file to remote server when running `attachFile`.
-   `restart`: (optional, default: true) - restart browser between tests.
-   `smartWait`: (optional) **enables [SmartWait][3]**; wait for additional milliseconds for element to appear. Enable for 5 secs: "smartWait": 5000.
-   `disableScreenshots`: (optional, default: false) - don't save screenshots on failure.
-   `fullPageScreenshots` (optional, default: false) - make full page screenshots on failure.
-   `uniqueScreenshotNames`: (optional, default: false) - option to prevent screenshot override if you have scenarios with the same name in different suites.
-   `keepBrowserState`: (optional, default: false) - keep browser state between tests when `restart` is set to false.
-   `keepCookies`: (optional, default: false) - keep cookies between tests when `restart` set to false.
-   `windowSize`: (optional) default window size. Set to `maximize` or a dimension in the format `640x480`.
-   `waitForTimeout`: (optional, default: 1000) sets default wait time in _ms_ for all `wait*` functions.
-   `desiredCapabilities`: Selenium's [desired
    capabilities][4].
-   `manualStart`: (optional, default: false) - do not start browser before a test, start it manually inside a helper
    with `this.helpers["WebDriver"]._startBrowser()`.
-   `timeouts`: [WebDriver timeouts][5] defined as hash.

Example:

```json
{
   "helpers": {
     "WebDriver" : {
       "smartWait": 5000,
       "browser": "chrome",
       "restart": false,
       "windowSize": "maximize",
       "timeouts": {
         "script": 60000,
         "page load": 10000
       }
     }
   }
}
```

Additional configuration params can be used from [webdriverio
website][6].

### Headless Chrome

```json
{
   "helpers": {
     "WebDriver" : {
       "url": "http://localhost",
       "browser": "chrome",
       "desiredCapabilities": {
         "chromeOptions": {
           "args": [ "--headless", "--disable-gpu", "--window-size=800,600" ]
         }
       }
     }
   }
}
```

### Internet Explorer

Additional configuration params can be used from [IE options][7]

```json
{
   "helpers": {
     "WebDriver" : {
       "url": "http://localhost",
       "browser": "internet explorer",
       "desiredCapabilities": {
         "ieOptions": {
           "ie.browserCommandLineSwitches": "-private",
           "ie.usePerProcessProxy": true,
           "ie.ensureCleanSession": true,
         }
       }
     }
   }
}
```

### Selenoid Options

```json
{
   "helpers": {
     "WebDriver" : {
       "url": "http://localhost",
       "browser": "chrome",
       "desiredCapabilities": {
         "selenoidOptions": {
           "enableVNC": true,
         }
       }
     }
   }
}
```

### Connect through proxy

CodeceptJS also provides flexible options when you want to execute tests to Selenium servers through proxy. You will
need to update the `helpers.WebDriver.capabilities.proxy` key.

```js
{
    "helpers": {
        "WebDriver": {
            "capabilities": {
                "proxy": {
                    "proxyType": "manual|pac",
                    "proxyAutoconfigUrl": "URL TO PAC FILE",
                    "httpProxy": "PROXY SERVER",
                    "sslProxy": "PROXY SERVER",
                    "ftpProxy": "PROXY SERVER",
                    "socksProxy": "PROXY SERVER",
                    "socksUsername": "USERNAME",
                    "socksPassword": "PASSWORD",
                    "noProxy": "BYPASS ADDRESSES"
                }
            }
        }
    }
}
```

For example,

```js
{
    "helpers": {
        "WebDriver": {
            "capabilities": {
                "proxy": {
                    "proxyType": "manual",
                    "httpProxy": "http://corporate.proxy:8080",
                    "socksUsername": "codeceptjs",
                    "socksPassword": "secret",
                    "noProxy": "127.0.0.1,localhost"
                }
            }
        }
    }
}
```

Please refer to [Selenium - Proxy Object][4] for more
information.

### Cloud Providers

WebDriver makes it possible to execute tests against services like `Sauce Labs` `BrowserStack` `TestingBot`
Check out their documentation on [available parameters][8]

Connecting to `BrowserStack` and `Sauce Labs` is simple. All you need to do
is set the `user` and `key` parameters. WebDriver automatically know which
service provider to connect to.

```js
{
    "helpers":{
        "WebDriver": {
            "url": "YOUR_DESIRED_HOST",
            "user": "YOUR_BROWSERSTACK_USER",
            "key": "YOUR_BROWSERSTACK_KEY",
            "capabilities": {
                "browserName": "chrome",

                // only set this if you're using BrowserStackLocal to test a local domain
                // "browserstack.local": true,

                // set this option to tell browserstack to provide addition debugging info
                // "browserstack.debug": true,
            }
        }
    }
}
```

#### SauceLabs

SauceLabs can be configured via wdio service, which should be installed additionally:

    npm i @wdio/sauce-service --save

It is important to make sure it is compatible with current webdriverio version.

Enable `wdio` plugin in plugins list and add `sauce` service:

```js
plugins: {
   wdio: {
       services: ['sauce'],
       user: ... ,// saucelabs username
       key: ... // saucelabs api key
       // additional config, from sauce service
   }
}
```

See [complete reference on webdriver.io][9].

> Alternatively, use [codeceptjs-saucehelper][10] for better reporting.

#### BrowserStack

BrowserStack can be configured via wdio service, which should be installed additionally:

    npm i @wdio/browserstack-service --save

It is important to make sure it is compatible with current webdriverio version.

Enable `wdio` plugin in plugins list and add `browserstack` service:

```js
plugins: {
   wdio: {
       services: ['browserstack'],
       user: ... ,// browserstack username
       key: ... // browserstack api key
       // additional config, from browserstack service
   }
}
```

See [complete reference on webdriver.io][11].

> Alternatively, use [codeceptjs-bshelper][12] for better reporting.

### Multiremote Capabilities

This is a work in progress but you can control two browsers at a time right out of the box.
Individual control is something that is planned for a later version.

Here is the [webdriverio docs][13] on the subject

```js
{
    "helpers": {
        "WebDriver": {
            "multiremote": {
                "MyChrome": {
                    "desiredCapabilities": {
                        "browserName": "chrome"
                     }
                },
                "MyFirefox": {
                   "desiredCapabilities": {
                       "browserName": "firefox"
                   }
                }
            }
        }
    }
}
```

## Access From Helpers

Receive a WebDriver client from a custom helper by accessing `browser` property:

```js
this.helpers['WebDriver'].browser
```

## Methods

### Parameters

-   `config`  

### \_locate

Get elements by different locator types, including strict locator.
Should be used in custom helpers:

```js
this.helpers['WebDriver']._locate({name: 'password'}).then //...
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `smartWait`   (optional, default `false`)

### \_locateCheckable

Find a checkbox by providing human readable text:

```js
this.helpers['WebDriver']._locateCheckable('I agree with terms and conditions').then // ...
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.

### \_locateClickable

Find a clickable element by providing human readable text:

```js
this.helpers['WebDriver']._locateClickable('Next page').then // ...
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.

### \_locateFields

Find field elements by providing human readable text:

```js
this.helpers['WebDriver']._locateFields('Your email').then // ...
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.

### acceptPopup

Accepts the active JavaScript native popup window, as created by window.alert|window.confirm|window.prompt.
Don't confuse popups with modal windows, as created by [various
libraries][14].

-   _Appium_: supported only for web testing

### amOnPage

Opens a web page in a browser. Requires relative or absolute url.
If url starts with `/`, opens a web page of a site defined in `url` config parameter.

```js
I.amOnPage('/'); // opens main page of website
I.amOnPage('https://github.com'); // opens github
I.amOnPage('/login'); // opens a login page
```

#### Parameters

-   `url`  url path or global url.-   _Appium_: supported only for web testing

### appendField

Appends text to a input field or textarea.
Field is located by name, label, CSS or XPath

```js
I.appendField('#myTextField', 'appended');
```

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator
-   `value`  text value to append.-   _Appium_: supported, but it's clear a field before insert in apps

### attachFile

Attaches a file to element located by label, name, CSS or XPath
Path to file is relative current codecept directory (where codecept.json or codecept.conf.js is located).
File will be uploaded to remote system (if tests are running remotely).

```js
I.attachFile('Avatar', 'data/avatar.jpg');
I.attachFile('form input[name=avatar]', 'data/avatar.jpg');
```

#### Parameters

-   `locator`  field located by label|name|CSS|XPath|strict locator.
-   `pathToFile`  local file path relative to codecept.json config file.
    Appium: not tested

### cancelPopup

Dismisses the active JavaScript popup, as created by window.alert|window.confirm|window.prompt.

-   _Appium_: supported only for web testing

### checkOption

Selects a checkbox or radio button.
Element is located by label or name or CSS or XPath.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
I.checkOption('#agree');
I.checkOption('I Agree to Terms and Conditions');
I.checkOption('agree', '//form');
```

#### Parameters

-   `field`  checkbox located by label | name | CSS | XPath | strict locator.
-   `context`  (optional) element located by CSS | XPath | strict locator.
    Appium: not tested

### clearCookie

Clears a cookie by name,
if none provided clears all cookies.

```js
I.clearCookie();
I.clearCookie('test');
```

#### Parameters

-   `cookie`  (optional) cookie name.-   _Appium_: supported only for web testing

### clearField

Clears a `<textarea>` or text `<input>` element's value.

```js
I.clearField('Email');
I.clearField('user[email]');
I.clearField('#email');
```

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator.-   _Appium_: supported

### click

Perform a click on a link or a button, given by a locator.
If a fuzzy locator is given, the page will be searched for a button, link, or image matching the locator string.
For buttons, the "value" attribute, "name" attribute, and inner text are searched. For links, the link text is searched.
For images, the "alt" attribute and inner text of any parent links are searched.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
// simple link
I.click('Logout');
// button of form
I.click('Submit');
// CSS button
I.click('#form input[type=submit]');
// XPath
I.click('//form/*[@type=submit]');
// link in context
I.click('Logout', '#nav');
// using strict locator
I.click({css: 'nav a.login'});
```

#### Parameters

-   `locator`  clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context`  (optional) element to search in CSS|XPath|Strict locator.-   _Appium_: supported

### closeCurrentTab

Close current tab.

```js
I.closeCurrentTab();
```

### closeOtherTabs

Close all tabs except for the current one.

-   _Appium_: supported web test

```js
I.closeOtherTabs();
```

### defineTimeout

Set [WebDriver timeouts][15] in realtime.

-   _Appium_: supported only for web testing.
    Timeouts are expected to be passed as object:

```js
I.defineTimeout({ script: 5000 });
I.defineTimeout({ implicit: 10000, pageLoad: 10000, script: 5000 });
```

#### Parameters

-   `timeouts`  WebDriver timeouts object.

### dontSee

Opposite to `see`. Checks that a text is not present on a page.
Use context parameter to narrow down the search.

```js
I.dontSee('Login'); // assume we are already logged in
```

#### Parameters

-   `text`  is not present.
-   `context`  (optional) element located by CSS|XPath|strict locator in which to perfrom search.-   _Appium_: supported with context in apps

### dontSeeCheckboxIsChecked

Verifies that the specified checkbox is not checked.

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator.
    Appium: not tested

### dontSeeCookie

Checks that cookie with given name does not exist.

#### Parameters

-   `name`  cookie name.-   _Appium_: supported only for web testing

### dontSeeCurrentUrlEquals

Checks that current url is not equal to provided one.
If a relative url provided, a configured url will be prepended to it.

#### Parameters

-   `url`  value to check.-   _Appium_: supported only for web testing

### dontSeeElement

Opposite to `seeElement`. Checks that element is not visible (or in DOM)

#### Parameters

-   `locator`  located by CSS|XPath|Strict locator.-   _Appium_: supported

### dontSeeElementInDOM

Opposite to `seeElementInDOM`. Checks that element is not on page.

#### Parameters

-   `locator`  located by CSS|XPath|Strict locator.-   _Appium_: supported

### dontSeeInCurrentUrl

Checks that current url does not contain a provided fragment.

#### Parameters

-   `url`  value to check.-   _Appium_: supported only for web testing

### dontSeeInField

Checks that value of input field or textare doesn't equal to given value
Opposite to `seeInField`.

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator.
-   `value`  value to check.-   _Appium_: supported only for web testing

### dontSeeInSource

Checks that the current page does not contains the given string in its raw source code.

#### Parameters

-   `text`  value to check.-   _Appium_: supported

### dontSeeInTitle

Checks that title does not contain text.

#### Parameters

-   `text`  text value to check.-   _Appium_: supported only for web testing

### doubleClick

Performs a double-click on an element matched by link|button|label|CSS or XPath.
Context can be specified as second parameter to narrow search.

```js
I.doubleClick('Edit');
I.doubleClick('Edit', '.actions');
I.doubleClick({css: 'button.accept'});
I.doubleClick('.btn.edit');
```

#### Parameters

-   `locator`  clickable link or button located by text, or any element located by CSS|XPath|strict locator.
-   `context`  (optional) element to search in CSS|XPath|Strict locator.-   _Appium_: supported only for web testing

### dragAndDrop

Drag an item to a destination element.

```js
I.dragAndDrop('#dragHandle', '#container');
```

#### Parameters

-   `srcElement`  located by CSS|XPath|strict locator.
-   `destElement`  located by CSS|XPath|strict locator.
    Appium: not tested

### dragSlider

Drag the scrubber of a slider to a given position
For fuzzy locators, fields are matched by label text, the "name" attribute, CSS, and XPath.

```js
I.dragSlider('#slider', 30);
I.dragSlider('#slider', -70);
```

#### Parameters

-   `locator`  located by label|name|CSS|XPath|strict locator.
-   `offsetX`  position to drag.

### executeAsyncScript

Executes async script on page.
Provided function should execute a passed callback (as first argument) to signal it is finished.

Example: In Vue.js to make components completely rendered we are waiting for [nextTick][16].

```js
I.executeAsyncScript(function(done) {
Vue.nextTick(done); // waiting for next tick
});
```

By passing value to `done()` function you can return values.
Additional arguments can be passed as well, while `done` function is always last parameter in arguments list.

```js
let val = await I.executeAsyncScript(function(url, done) {
// in browser context
$.ajax(url, { success: (data) => done(data); }
}, 'http://ajax.callback.url/');
```

#### Parameters

-   `fn`  function to be executed in browser context.

### executeScript

Executes sync script on a page.
Pass arguments to function as additional parameters.
Will return execution result to a test.
In this case you should use async function and await to receive results.

Example with jQuery DatePicker:

```js
// change date of jQuery DatePicker
I.executeScript(function() {
// now we are inside browser context
$('date').datetimepicker('setDate', new Date());
});
```

Can return values. Don't forget to use `await` to get them.

```js
let date = await I.executeScript(function(el) {
// only basic types can be returned
return $(el).datetimepicker('getDate').toString();
}, '#date'); // passing jquery selector
```

#### Parameters

-   `fn`  function to be executed in browser context.

### fillField

Fills a text field or textarea, after clearing its value, with the given string.
Field is located by name, label, CSS, or XPath.

```js
// by label
I.fillField('Email', 'hello@world.com');
// by name
I.fillField('password', secret('123456'));
// by CSS
I.fillField('form#login input[name=username]', 'John');
// or by strict locator
I.fillField({css: 'form#login input[name=username]'}, 'John');
```

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator.
-   `value`  text value to fill.-   _Appium_: supported

### grabAttributeFrom

Retrieves an attribute from an element located by CSS or XPath and returns it to test.
An array as a result will be returned if there are more than one matched element.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let hint = await I.grabAttributeFrom('#tooltip', 'title');
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `attr`  attribute name.
    Appium: can be used for apps only with several values ("contentDescription", "text", "className", "resourceId")

### grabBrowserLogs

Get JS log from browser. Log buffer is reset after each request.

```js
let logs = await I.grabBrowserLogs();
console.log(JSON.stringify(logs))
```

### grabCookie

Gets a cookie object by name.
If none provided gets all cookies.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let cookie = await I.grabCookie('auth');
assert(cookie.value, '123456');
```

#### Parameters

-   `name`  (optional) cookie name.-   _Appium_: supported only for web testing

### grabCssPropertyFrom

Grab CSS property for given locator
Resumes test execution, so **should be used inside an async function with `await`** operator.

```js
const value = await I.grabCssPropertyFrom('h3', 'font-weight');
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `cssProperty`  CSS property name.

### grabCurrentUrl

Get current URL from browser.
Resumes test execution, so should be used inside an async function.

```js
let url = await I.grabCurrentUrl();
console.log(`Current URL is [${url}]`);
```

### grabHTMLFrom

Retrieves the innerHTML from an element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.
If more than one element is found - an array of HTMLs returned.

```js
let postHTML = await I.grabHTMLFrom('#post');
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.-   _Appium_: supported only for web testing

### grabNumberOfOpenTabs

Grab number of open tabs.

```js
I.grabNumberOfOpenTabs();
```

### grabNumberOfVisibleElements

Grab number of visible elements by locator.

```js
I.grabNumberOfVisibleElements('p');
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.

### grabPageScrollPosition

Retrieves a page scroll position and returns it to test.
Resumes test execution, so **should be used inside an async function with `await`** operator.

```js
let { x, y } = await I.grabPageScrollPosition();
```

### grabPopupText

Grab the text within the popup. If no popup is visible then it will return null.

```js
await I.grabPopupText();
```

### grabSource

Retrieves page source and returns it to test.
Resumes test execution, so should be used inside an async function.

```js
let pageSource = await I.grabSource();
```

-   _Appium_: supported

### grabTextFrom

Retrieves a text from an element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let pin = await I.grabTextFrom('#pin');
```

If multiple elements found returns an array of texts.

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.-   _Appium_: supported

### grabTitle

Retrieves a page title and returns it to test.
Resumes test execution, so **should be used inside async with `await`** operator.

```js
let title = await I.grabTitle();
```

-   _Appium_: supported only for web testing

### grabValueFrom

Retrieves a value from a form element located by CSS or XPath and returns it to test.
Resumes test execution, so **should be used inside async function with `await`** operator.

```js
let email = await I.grabValueFrom('input[name=email]');
```

#### Parameters

-   `locator`  field located by label|name|CSS|XPath|strict locator.-   _Appium_: supported only for web testing

### moveCursorTo

Moves cursor to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.moveCursorTo('.tooltip');
I.moveCursorTo('#submit', 5,5);
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.
-   `offsetX`  (optional) X-axis offset.
-   `offsetY`  (optional) Y-axis offset.-   _Appium_: supported only for web testing

### openNewTab

Open new tab and switch to it.

```js
I.openNewTab();
```

### pressKey

Presses a key on a focused element.
Special keys like 'Enter', 'Control', [etc][17]
will be replaced with corresponding unicode.
If modifier key is used (Control, Command, Alt, Shift) in array, it will be released afterwards.

```js
I.pressKey('Enter');
I.pressKey(['Control','a']);
```

#### Parameters

-   `key`  key or array of keys to press.
    [Valid key names][18] are:-   `'Add'`,
    -   `'Alt'`,
    -   `'ArrowDown'` or `'Down arrow'`,
    -   `'ArrowLeft'` or `'Left arrow'`,
    -   `'ArrowRight'` or `'Right arrow'`,
    -   `'ArrowUp'` or `'Up arrow'`,
    -   `'Backspace'`,
    -   `'Command'`,
    -   `'Control'`,
    -   `'Del'`,
    -   `'Divide'`,
    -   `'End'`,
    -   `'Enter'`,
    -   `'Equals'`,
    -   `'Escape'`,
    -   `'F1 to F12'`,
    -   `'Home'`,
    -   `'Insert'`,
    -   `'Meta'`,
    -   `'Multiply'`,
    -   `'Numpad 0'` to `'Numpad 9'`,
    -   `'Pagedown'` or `'PageDown'`,
    -   `'Pageup'` or `'PageUp'`,
    -   `'Pause'`,
    -   `'Semicolon'`,
    -   `'Shift'`,
    -   `'Space'`,
    -   `'Subtract'`,
    -   `'Tab'`.To make combinations with modifier and mouse clicks (like Ctrl+Click) press a modifier, click, then release it.-   _Appium_: supported, but clear field before pressing in apps:```js
    I.pressKey('Control');
    I.click('#someelement');
    I.pressKey('Control');
    ```

### refreshPage

Reload the current page.

```js
`I.refreshPage();
```

### resizeWindow

Resize the current window to provided width and height.
First parameter can be set to `maximize`.

#### Parameters

-   `width`  width in pixels or `maximize`.
-   `height`  height in pixels.
    Appium: not tested in web, in apps doesn't work

### rightClick

Performs right click on a clickable element matched by semantic locator, CSS or XPath.

```js
// right click element with id el
I.rightClick('#el');
// right click link or button with text "Click me"
I.rightClick('Click me');
// right click button with text "Click me" inside .context
I.rightClick('Click me', '.context');
```

#### Parameters

-   `locator`  clickable element located by CSS|XPath|strict locator.
-   `context`  (optional) element located by CSS|XPath|strict locator.-   _Appium_: supported, but in apps works as usual click

### runInWeb

Placeholder for ~ locator only test case write once run on both Appium and WebDriver.

#### Parameters

-   `fn`  

### runOnAndroid

Placeholder for ~ locator only test case write once run on both Appium and WebDriver.

#### Parameters

-   `caps`  
-   `fn`  

### runOnIOS

Placeholder for ~ locator only test case write once run on both Appium and WebDriver.

#### Parameters

-   `caps`  
-   `fn`  

### saveScreenshot

Saves a screenshot to ouput folder (set in codecept.json or codecept.conf.js).
Filename is relative to output folder. 
Optionally resize the window to the full available page `scrollHeight` and `scrollWidth` to capture the entire page by passing `true` in as the second argument.

```js
I.saveScreenshot('debug.png');
I.saveScreenshot('debug.png', true) //resizes to available scrollHeight and scrollWidth before taking screenshot
```

#### Parameters

-   `fileName`  file name to save.
-   `fullPage`  (optional) flag to enable fullscreen screenshot mode.-   _Appium_: supported

### scrollPageToBottom

Scroll page to the bottom.

```js
I.scrollPageToBottom();
```

### scrollPageToTop

Scroll page to the top.

```js
I.scrollPageToTop();
```

### scrollTo

Scrolls to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.scrollTo('footer');
I.scrollTo('#submit', 5, 5);
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.
-   `offsetX`  (optional) X-axis offset.
-   `offsetY`  (optional) Y-axis offset.-   _Appium_: supported only for web testing

### scrollTo

Scrolls to element matched by locator.
Extra shift can be set with offsetX and offsetY options.

```js
I.scrollTo('footer');
I.scrollTo('#submit', 5, 5);
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.
-   `offsetX`  (optional) X-axis offset.
-   `offsetY`  (optional) Y-axis offset.

### see

Checks that a page contains a visible text.
Use context parameter to narrow down the search.

```js
I.see('Welcome'); // text welcome on a page
I.see('Welcome', '.content'); // text inside .content div
I.see('Register', {css: 'form.register'}); // use strict locator
```

#### Parameters

-   `text`  expected on page.
-   `context`  (optional) element located by CSS|Xpath|strict locator in which to search for text.-   _Appium_: supported with context in apps

### seeAttributesOnElements

Checks that all elements with given locator have given attributes.

```js
I.seeAttributesOnElements('//form', {'method': "post"});
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.
-   `attributes`  object with attributes and their values to check.

### seeCheckboxIsChecked

Verifies that the specified checkbox is checked.

```js
I.seeCheckboxIsChecked('Agree');
I.seeCheckboxIsChecked('#agree'); // I suppose user agreed to terms
I.seeCheckboxIsChecked({css: '#signup_form input[type=checkbox]'});
```

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator.
    Appium: not tested

### seeCookie

Checks that cookie with given name exists.

```js
I.seeCookie('Auth');
```

#### Parameters

-   `name`  cookie name.-   _Appium_: supported only for web testing

### seeCssPropertiesOnElements

Checks that all elements with given locator have given CSS properties.

```js
I.seeCssPropertiesOnElements('h3', { 'font-weight': "bold"});
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.
-   `cssProperties`  object with CSS properties and their values to check.

### seeCurrentUrlEquals

Checks that current url is equal to provided one.
If a relative url provided, a configured url will be prepended to it.
So both examples will work:

```js
I.seeCurrentUrlEquals('/register');
I.seeCurrentUrlEquals('http://my.site.com/register');
```

#### Parameters

-   `url`  value to check.-   _Appium_: supported only for web testing

### seeElement

Checks that a given Element is visible
Element is located by CSS or XPath.

```js
I.seeElement('#modal');
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.-   _Appium_: supported

### seeElementInDOM

Checks that a given Element is present in the DOM
Element is located by CSS or XPath.

```js
I.seeElementInDOM('#modal');
```

#### Parameters

-   `locator`  located by CSS|XPath|strict locator.-   _Appium_: supported

### seeInCurrentUrl

Checks that current url contains a provided fragment.

```js
I.seeInCurrentUrl('/register'); // we are on registration page
```

#### Parameters

-   `url`  value to check.-   _Appium_: supported only for web testing

### seeInField

Checks that the given input field or textarea equals to given value.
For fuzzy locators, fields are matched by label text, the "name" attribute, CSS, and XPath.

```js
I.seeInField('Username', 'davert');
I.seeInField({css: 'form textarea'},'Type your comment here');
I.seeInField('form input[type=hidden]','hidden_value');
I.seeInField('#searchform input','Search');
```

#### Parameters

-   `field`  located by label|name|CSS|XPath|strict locator.
-   `value`  value to check.-   _Appium_: supported only for web testing

### seeInPopup

Checks that the active JavaScript popup, as created by `window.alert|window.confirm|window.prompt`, contains the
given string.

-   _Appium_: supported only for web testing

#### Parameters

-   `text`  value to check.

### seeInSource

Checks that the current page contains the given string in its raw source code.

```js
I.seeInSource('<h1>Green eggs &amp; ham</h1>');
```

#### Parameters

-   `text`  value to check.-   _Appium_: supported

### seeInTitle

Checks that title contains text.

#### Parameters

-   `text`  text value to check.-   _Appium_: supported only for web testing

### seeNumberOfElements

Asserts that an element appears a given number of times in the DOM.
Element is located by label or name or CSS or XPath.

-   _Appium_: supported

```js
I.seeNumberOfElements('#submitBtn', 1);
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `num`  number of elements.

### seeNumberOfVisibleElements

Asserts that an element is visible a given number of times.
Element is located by CSS or XPath.

```js
I.seeNumberOfVisibleElements('.buttons', 3);
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `num`  number of elements.

### seeTextEquals

Checks that text is equal to provided one.

```js
I.seeTextEquals('text', 'h1');
```

#### Parameters

-   `text`  element value to check.
-   `context`  (optional) element located by CSS|XPath|strict locator.

### seeTitleEquals

Checks that title is equal to provided one.

```js
I.seeTitleEquals('Test title.');
```

#### Parameters

-   `text`  value to check.

### selectOption

Selects an option in a drop-down select.
Field is searched by label | name | CSS | XPath.
Option is selected by visible text or by value.

```js
I.selectOption('Choose Plan', 'Monthly'); // select by label
I.selectOption('subscription', 'Monthly'); // match option by text
I.selectOption('subscription', '0'); // or by value
I.selectOption('//form/select[@name=account]','Premium');
I.selectOption('form select[name=account]', 'Premium');
I.selectOption({css: 'form select[name=account]'}, 'Premium');
```

Provide an array for the second argument to select multiple options.

```js
I.selectOption('Which OS do you use?', ['Android', 'iOS']);
```

#### Parameters

-   `select`  field located by label|name|CSS|XPath|strict locator.
-   `option`  visible text or value of option.

### setCookie

Sets a cookie.

```js
I.setCookie({name: 'auth', value: true});
```

#### Parameters

-   `cookie`  cookie JSON object.-   _Appium_: supported only for web testingUses Selenium's JSON [cookie
    format][19].

### switchTo

Switches frame or in case of null locator reverts to parent.

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.-   _Appium_: supported only for web testing

### switchToNextTab

Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.

```js
I.switchToNextTab();
I.switchToNextTab(2);
```

#### Parameters

-   `num`  (optional) number of tabs to switch forward, default: 1.
-   `sec`  (optional) time in seconds to wait.

### switchToPreviousTab

Switch focus to a particular tab by its number. It waits tabs loading and then switch tab.

```js
I.switchToPreviousTab();
I.switchToPreviousTab(2);
```

#### Parameters

-   `num`  (optional) number of tabs to switch backward, default: 1.
-   `sec`  (optional) time in seconds to wait.

### uncheckOption

Unselects a checkbox or radio button.
Element is located by label or name or CSS or XPath.

The second parameter is a context (CSS or XPath locator) to narrow the search.

```js
I.uncheckOption('#agree');
I.uncheckOption('I Agree to Terms and Conditions');
I.uncheckOption('agree', '//form');
```

#### Parameters

-   `field`  checkbox located by label | name | CSS | XPath | strict locator.
-   `context`  (optional) element located by CSS | XPath | strict locator.
    Appium: not tested

### wait

Pauses execution for a number of seconds.

```js
I.wait(2); // wait 2 secs
```

#### Parameters

-   `sec`  time in seconds to wait.-   _Appium_: supported

### waitForDetached

Waits for an element to become not attached to the DOM on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

    I.waitForDetached('#popup');

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `sec`  (optional) time in seconds to wait, 1 by default.-   _Appium_: supported

### waitForElement

Waits for element to be present on page (by default waits for 1sec).
Element can be located by CSS or XPath.

```js
I.waitForElement('.btn.continue');
I.waitForElement('.btn.continue', 5); // wait for 5 secs
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `sec`  (optional) time in seconds to wait, 1 by default.

### waitForEnabled

Waits for element to become enabled (by default waits for 1sec).
Element can be located by CSS or XPath.

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `sec`  (optional) time in seconds to wait, 1 by default.-   _Appium_: supported

### waitForFunction

Waits for a function to return true (waits for 1 sec by default).
Running in browser context.

```js
I.waitForFunction(fn[, [args[, timeout]])
```

```js
I.waitForFunction(() => window.requests == 0);
I.waitForFunction(() => window.requests == 0, 5); // waits for 5 sec
I.waitForFunction((count) => window.requests == count, [3], 5) // pass args and wait for 5 sec
```

#### Parameters

-   `fn`  to be executed in browser context.
-   `argsOrSec`  (optional) arguments for function or seconds.
-   `sec`  (optional) time in seconds to wait, 1 by default.-   _Appium_: supported

### waitForInvisible

Waits for an element to be removed or become invisible on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

    I.waitForInvisible('#popup');

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `sec`  (optional) time in seconds to wait, 1 by default.-   _Appium_: supported

### waitForText

Waits for a text to appear (by default waits for 1sec).
Element can be located by CSS or XPath.
Narrow down search results by providing context.

```js
I.waitForText('Thank you, form has been submitted');
I.waitForText('Thank you, form has been submitted', 5, '#modal');
```

#### Parameters

-   `text`  to wait for.
-   `sec`  (optional) time in seconds to wait.
-   `context`  (optional) element located by CSS|XPath|strict locator.

### waitForValue

Waits for the specified value to be in value attribute.

```js
I.waitForValue('//input', "GoodValue");
```

#### Parameters

-   `field`  input field.
-   `value`  expected value.
-   `sec`  (optional) time in seconds to wait, 1 sec by default.

### waitForVisible

Waits for an element to become visible on a page (by default waits for 1sec).
Element can be located by CSS or XPath.

    I.waitForVisible('#popup');

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `sec`  (optional) time in seconds to wait, 1 by default.-   _Appium_: supported

### waitInUrl

Waiting for the part of the URL to match the expected. Useful for SPA to understand that page was changed.

```js
I.waitInUrl('/info', 2);
```

#### Parameters

-   `urlPart`  value to check.
-   `sec`  (optional) time in seconds to wait.

### waitNumberOfVisibleElements

Waits for a specified number of elements on the page.

```js
I.waitNumberOfVisibleElements('a', 3);
```

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `num`  number of elements.
-   `sec`  (optional) time in seconds to wait.

### waitToHide

Waits for an element to hide (by default waits for 1sec).
Element can be located by CSS or XPath.

    I.waitToHide('#popup');

#### Parameters

-   `locator`  element located by CSS|XPath|strict locator.
-   `sec`  (optional) time in seconds to wait, 1 by default.-   _Appium_: supported

### waitUntil

Waits for a function to return true (waits for 1sec by default).

```js
I.waitUntil(() => window.requests == 0);
I.waitUntil(() => window.requests == 0, 5);
```

#### Parameters

-   `fn`  function which is executed in browser context.
-   `sec`  (optional) time in seconds to wait, 1 by default.
-   `timeoutMsg`  (optional) message to show in case of timeout fail.
-   `interval`  (optional) time in seconds between condition checks.-   _Appium_: supported

### waitUrlEquals

Waits for the entire URL to match the expected

```js
I.waitUrlEquals('/info', 2);
I.waitUrlEquals('http://127.0.0.1:8000/info');
```

#### Parameters

-   `urlPart`  value to check.
-   `sec`  (optional) time in seconds to wait.

[1]: http://webdriver.io/

[2]: http://codecept.io/quickstart/#prepare-selenium-server

[3]: http://codecept.io/acceptance/#smartwait

[4]: https://github.com/SeleniumHQ/selenium/wiki/DesiredCapabilities

[5]: http://webdriver.io/docs/timeouts.html

[6]: http://webdriver.io/guide/getstarted/configuration.html

[7]: https://seleniumhq.github.io/selenium/docs/api/rb/Selenium/WebDriver/IE/Options.html

[8]: http://webdriver.io/guide/usage/cloudservices.html

[9]: https://webdriver.io/docs/sauce-service.html

[10]: https://github.com/puneet0191/codeceptjs-saucehelper/

[11]: https://webdriver.io/docs/browserstack-service.html

[12]: https://github.com/PeterNgTr/codeceptjs-bshelper

[13]: http://webdriver.io/guide/usage/multiremote.html

[14]: http://jster.net/category/windows-modals-popups

[15]: https://webdriver.io/docs/timeouts.html

[16]: https://vuejs.org/v2/api/#Vue-nextTick

[17]: https://code.google.com/p/selenium/wiki/JsonWireProtocol#/session/:sessionId/element/:id/value

[18]: https://w3c.github.io/webdriver/#keyboard-actions

[19]: https://code.google.com/p/selenium/wiki/JsonWireProtocol#Cookie_JSON_Object
