---
id: quickstart
title: Quickstart
---

**NodeJS v 8.9** and higher required to start.
CodeceptJS is multi-backend testing framework. It can execute tests using different libraries like webdriverio, Puppeteer, Protractor, etc.

* In this guide we will use [Google Chrome **Puppeteer**](https://github.com/GoogleChrome/puppeteer) as a driver for browsers. This allows us to start in a minutes with no extra tools installed.
* If you are familiar with Selenium, you can choose classical [**Selenium WebDriver** setup](#using-selenium-webdriver).
* Also, look at [complete installation reference](https://codecept.io/installation/).


## Using Puppeteer


<video onclick="this.paused ? this.play() : this.pause();" src="/img/codeceptjs-install.mp4" style="width: 100%" controls></video>


1) Install CodeceptJS with Puppeteer

```
npm install codeceptjs puppeteer --save-dev
```


2) Initialize CodeceptJS in current directory by running:

```sh
npx codeceptjs init
```

(use `node node_modules/.bin/codeceptjs` if you have issues with npx)

3) Answer questions. Agree on defaults, when asked to select helpers choose **Puppeteer**.

```sh
? What helpers do you want to use?
 ◯ WebDriver
 ◯ Protractor
❯◉ Puppeteer
 ◯ Appium
 ◯ Nightmare
 ◯ FileSystem
```

4) Create First Test.

```bash
npx codeceptjs gt
```

5) Enter a test name. Open a generated file in your favorite JavaScript editor.

```js
Feature('My First Test');

Scenario('test something', (I) => {

});
```

6) Write a simple scenario

```js
Feature('My First Test');

Scenario('test something', (I) => {
  I.amOnPage('https://github.com');
  I.see('GitHub');
});
```

7) Run a test:

```
npx codeceptjs run --steps
```

The output should be similar to this:

```bash
My First Test --
  test something
   • I am on page "https://github.com"
   • I see "GitHub"
 ✓ OK
```

Puppeteer starts a browser without showing its window. To see the browser, edit `codecept.json` config and set `show: true` for Puppeteer:

```js
{
  "helpers": {
    "Puppeteer": {
      "url": "http://localhost",
      "show": true,
    }
  }
}
```

Rerun the test to see the browser.

> Next: [CodeceptJS with Puppeteer >>>](https://codecept.io/puppeteer/)

> Next: [CodeceptJS Basics >>>](https://codecept.io/basics/)

> Next: [Demo Project](https://github.com/DavertMik/codeceptjs-todomvc-puppeteer)


---

## Using Selenium WebDriver

1) Install CodeceptJS with webdriverio library

```
npm install codeceptjs webdriverio --save-dev
```

2) Initialize CodeceptJS in current directory by running:

```sh
npx codeceptjs init
```

(use `node node_modules/.bin/codeceptjs init` if you have issues with npx)

3) Answer questions. Agree on defaults, when asked to select helpers choose **WebDriver**.

```sh
? What helpers do you want to use?
❯◉ WebDriver
 ◯ Protractor
 ◯ Puppeteer
 ◯ Appium
 ◯ Nightmare
 ◯ FileSystem
```

4) Create First Test.

```bash
npx codeceptjs gt
```

5) Enter a test name. Open a generated file in your favorite JavaScript editor.

```js
Feature('My First Test');

Scenario('test something', (I) => {

});
```

6) Write a simple scenario

```js
Feature('My First Test');

Scenario('test something', (I) => {
  I.amOnPage('https://github.com');
  I.see('GitHub');
});
```

7) Prepare Selenium Server

Install `@wdio/selenium-standalone-service` package to automatically start and stop selenium service.

```
npm i @wdio/selenium-standalone-service --save
```

Enable it in config inside plugins section:

```js
exports.config = {
  // ...
  // inside condecept.conf.js
  plugins: {
    wdio: {
        services: ['selenium-standalone']
    }
  }
}
```

> Alternatively, use [selenium-standalone](https://www.npmjs.com/package/selenium-standalone) to install, start and stop Selenium Server manually.


8) Run a test:

```
npx codeceptjs run --steps
```

If everything is done right, you will see in console:

```bash
My First Test --
  test something
   • I am on page "https://github.com"
   • I see "GitHub"
 ✓ OK
```


> Next: [CodeceptJS Basics >>>](https://codecept.io/basics/)

> Next: [Acceptance Testing in CodeceptJS >>>](https://codecept.io/acceptance/)


## Using Protractor

> [**Follow corresponding guide >>**](https://codecept.io/angular/)

## Using Appium

> [**Follow corresponding guide >>**](https://codecept.io/mobile/)

## Using NightmareJS

> [**Follow corresponding guide >>**](https://codecept.io/nightmare/)
